#! /usr/bin/env node

const yargs = require("yargs");
const path = require("path");
const fs = require('fs');
const boxen = require("boxen");
const { isNumber } = require('lodash');
const root = process.cwd();

const index = (fold) => 'const { API } = require("easy-api.ts");'
    + "\n\nconst api = new API({"
    + "\n    logger: false,"
    + "\n    reverse: false"
    + "\n});\n"
    + "\napi.route({"
    + '\n    url: "/json/helloworld",'
    + '\n    method: "GET",'
    + "\n    code: \`"
    + "\n        $response: ["
    + "\n            $setCode: [200]"
    + "\n            $setType: [JSON]"
    + "\n            $setBody: [{"
    + "\n                \"message\": \"Hello world!\""
    + "\n            }]"
    + "\n        ]"
    + "\n    \`"
    + "\n});"
    + "\n\napi.load(\"./"+ fold + "\").then(() => console.log(\"Routes loaded!\"));"
    + "\napi.on(\"ready\", () => console.log(\"API ready!\"));";

const route = "module.exports[\"route\"] = {"
    + "\n    name: \"/json/color\","
    + "\n    method: \"GET\","
    + "\n    code: `"
    + "\n        $if: [$query:[hex]==null;"
    + "\n            $response: ["
    + "\n                $setCode:[400]"
    + "\n                $setType:[JSON]"
    + "\n                $setBody:[{"
    + "\n                    \"error\": \"Invalid hex color code provided!\""
    + "\n                }]"
    + "\n            ]"
    + "\n            $break"
    + "\n        ]"
    + "\n        $createCanvas: ["
    + "\n            $setDimentions: [512;512]"
    + "\n            $color: [$query: [hex]]"
    + "\n            $drawRect: [0;0;512;512]"
    + "\n        ]"
    + "\n        $response: ["
    + "\n            $setCode: [200]"
    + "\n            $setType: [canvas]"
    + "\n            $setBody: [$default]"
    + "\n        ]"
    + "\n    `"
    + "\n}";

async function createSetup(port, routePath) {
    let handler = true;
    // Creating index.js
    fs.writeFileSync(path.join(root, "/index.js"), index(routePath) + "\n\n" + `api.connect({ port: process.env["PORT"] ?? ${port} });`);

    // Creating handler files
    if (!fs.existsSync(path.join(root, `./${routePath}`)))
        fs.mkdirSync(path.join(root, `./${routePath}`));
    
    try {
        fs.writeFileSync(path.join(root, `./${routePath}/color.js`), route);
    } catch { handler = false };

    console.log("| ------------- SUCCESS ---------------|");
    console.log("|-> ✔️  Main file: \x1b[32mindex.js\x1b[0m");
    console.log("|-> " + `${handler ? "✔️  Handler: \x1b[32m" + path.join(root, routePath) + "\x1b[0m" : "❌ Handler: \x1b[31m Cannot be created.\x1b[0m"}`);
    console.log("| -------------------------------------|");
}

async function main() {
    const args = yargs.parse(process.argv.slice(2));
    if (
        args._.length === 0 &&
        !args.port &&
        !args.folder &&
        !args.canvas &&
        !args.json
    ) {
        console.clear();
        console.log(boxen("easy-api.ts: The most advanced string-based API maker.", {
            borderColor: "#3498DB",
            borderStyle: "round"
        }))
        console.log("Flags:");
        console.log("\x1b[34m--port\x1b[0m Set the API port.");
        console.log("\x1b[34m--folder\x1b[0m Set the route loader folder name.");
        console.log("\x1b[34m--canvas\x1b[0m Loads pre-bundled canvas routes.");
        console.log("\x1b[34m--json\x1b[0m Loads pre-bundled JSON routes.");
        return;
    }
    let port = 3000, rpath = "routes";
    if (args.port && isNumber(args.port)) port = args.port;
    if (args.folder && isNaN(args.folder)) rpath = args.folder;
    await createSetup(port, rpath);
    if (args.canvas) {
        const r = fs.readFileSync(path.join(__dirname, "prebundled/CANVAS_rankcard.js"), { encoding: "utf-8" });
        fs.writeFileSync(path.join(root, `${rpath}/rankcard.js`), r);
    }
    if (args.json) {
        const r = fs.readFileSync(path.join(__dirname, "prebundled/JSON_owoify.js"), { encoding: "utf-8" });
        fs.writeFileSync(path.join(root, `${rpath}/owoify.js`), r);
    }
}

main();