import { TypedEmitter } from "tiny-typed-emitter";
import { Interpreter, Data } from "./Data";
import { DatabaseManager } from "./DatabaseManager";
import * as F from "fastify";
export interface EATS_EVENTS {
    error: (exception: Error) => any;
    request: (data: Data) => any;
    ready: (data: Data) => any;
}
export interface EATS_ROUTE extends Record<string, any> {
    method: F.HTTPMethods;
    url: string;
    code: string;
    comments?: boolean;
}
export interface EATS_EVENT extends Record<string, any> {
    name: keyof EATS_EVENTS;
    code: string;
    comments?: boolean;
}
export declare class EATS_ERROR extends Error {
    metadata: Record<string, any>;
    constructor(message: string, metadata: Record<string, any>);
}
export interface EATS_API_OPTIONS extends F.FastifyServerOptions {
    dots?: boolean;
    reverse?: boolean;
    debug?: boolean;
}
export declare class API extends TypedEmitter<EATS_EVENTS> {
    private functions;
    app: F.FastifyInstance;
    interpreter: Interpreter;
    options: EATS_API_OPTIONS;
    db: DatabaseManager;
    private dots;
    constructor(ops: EATS_API_OPTIONS);
    route(data: EATS_ROUTE): Promise<void>;
    event(data: EATS_EVENT): void;
    load(dir: string): Promise<void>;
    private isEvent;
    private isRoute;
    connect(options: F.FastifyListenOptions): Promise<void>;
}
