import { FunctionManager } from "./manager.js";
import { TypedEmitter } from "tiny-typed-emitter";
export declare class BDJS_Error extends Error {
    data: Record<string, any>;
    constructor(message: string, data: Record<string, any>);
}
declare class Token {
    type: "Text" | "Function";
    content: string;
    constructor(type: "Text" | "Function", content: string);
}
export interface DataOptions {
    break: boolean;
    code: string;
    comments?: boolean;
    dots?: boolean;
    interpreter: Interpreter;
    start: number | string;
    functions: FunctionManager;
    func?: {
        name: string;
        inside: string;
        splits: string[];
    };
    _?: {
        __request: Map<string, Object>;
        __cases: Map<string, string>;
        __vars: Map<string, unknown>;
        __arrays: Map<string, Array<string | number>>;
        __objects: Map<string, Object>;
        __functions: Map<string, string>;
    };
    __reverse: boolean;
}
export declare class Data implements DataOptions {
    break: boolean;
    code: string;
    comments?: boolean;
    dots?: boolean;
    interpreter: Interpreter;
    start: number | string;
    functions: FunctionManager;
    func?: {
        name: string;
        inside: string;
        splits: string[];
    };
    _?: {
        __request: Map<string, Object>;
        __cases: Map<string, string>;
        __vars: Map<string, unknown>;
        __arrays: Map<string, Array<string | number>>;
        __objects: Map<string, Object>;
        __functions: Map<string, string>;
    };
    __reverse: boolean;
    constructor(options: DataOptions);
}
export type InterpreterOptions = {
    functionManager: FunctionManager;
};
export interface Events {
    error: (error: Error, meta?: Record<string, any>) => void;
}
export declare class Interpreter extends TypedEmitter<Events> {
    functions: FunctionManager;
    private _options;
    constructor(options: InterpreterOptions);
    /**
     * Split RAW BDJS code into tokens.
     * @param code BDJS code.
     * @param reverse Switch between reading modes. [TOP-BOTTOM / BOTTOM-TOP]
     * @returns
     */
    __split(code: string, reverse?: boolean, dots?: boolean): Token[];
    /**
     * Parses Interpreter#split token content.
     * @param code Token function content.
     */
    __parse(code: string): {
        name: string;
        inside: string;
        splits: string[];
    };
    /**
     * Executes BDJS code.
     * @param raw_code
     * @param reverse
     * @param data
     * @param type
     */
    run<T extends Data | undefined>(raw_code: string, reverse?: boolean, data?: T, comments?: boolean, dots?: boolean): Promise<NonNullable<T>>;
}
export {};
