import { DataOptions } from "./interpreter.js";
import { Func } from "./builder.js";
export type CachedFunction = {
    data: Func;
    code: <D extends DataOptions = DataOptions>(d: D, ...args: any) => any;
};
export declare class FunctionManager {
    cache: Map<string, CachedFunction>;
    /**
     * Load core functions.
     * @param {string} dir Functions path.
     */
    loadCoreFunctions(dir?: string): Promise<void>;
    /**
     * Load core functions.
     * @param {string} dir Functions path.
     */
    load(dir: string): Promise<void>;
    /**
     * Add a function to the case.
     * @param {CachedFunction}} fn Function.
     */
    add(fn: CachedFunction): void;
    /**
     * Get a function from cache.
     * @param name Function name.
     */
    get(name: string): CachedFunction | undefined;
    /**
     * Delete a function from cache.
     * @param name
     */
    delete(name: string): void;
    /**
     * Return all saved functions in an array.
     * @returns {CachedFunction[]}
     */
    toArray(): CachedFunction[];
}
